--------------------------------------------------------------------
DRealty - The Drupal Real Estate Framework
--------------------------------------------------------------------

DRealty is a real estate framework to import RETS resources (Property,
Agent, Office, OpenHouse, Media, etc.) into Drupal as entities. In D8/9 version of
DRealty, unlike D7 version, RETS data can be mapped into Nodes.

It's primary function is facilitating the bridge between getting the data
into the site and mapping fields between resources and Drupal.

From here, a user can setup Views for searching via the typical Views method or
using additional modules like ApacheSolr, SearchAPI and Faceted searching.

Drealty Drupal 8/9 version is still not a complete and fully tested solution for 
all the various RETS systems, but it's a good start for further improvements and fixes.

--------------------------------------------------------------------
System Requirements
--------------------------------------------------------------------

PHP requirements:
* php 7.4 - 8.0  http://www.php.net
* php cURL support - http://www.php.net/manual/en/curl.setup.php

Most hosting vendors come with this by default.

--------------------------------------------------------------------
Installation
--------------------------------------------------------------------

  - Download phRETS library with composer (https://github.com/troydavisson/PHRETS/tree/2.6.2)
  - Download and enable DRealty module (for basic functionality and Drush support)
  - Enable DRealty Reconciler module - to have DRealty entities checked against
      RETS system on CRON runs, to see if they still exist there and get
      deactivated/deleted if not. Its configuration will be available for each
      configured RETS connection when you edit it, under `/admin/drealty/connection/[connection_name]/edit`.
    For reconciler frequency "3 days" is recommended.

--------------------------------------------------------------------
General Configuration
--------------------------------------------------------------------

All DRealty needs to work is the connection credentials to your RETS provider.
Under DRealty (/admin/drealty/connection), you can add one or more connections
to your providers. 

Set "Modified Timestamp span" value for incremental updates. E.g. if you set it 
to 4 hours, then you'll need to have Drush import running every 3-4 hours. We would
recommend Drush to run every 3 hours in this case to not have any lost time spans
before it gets invoked.

Please note that all vendors operate on slightly varied implementations of RETS.
If all goes well, you are connected to your remote RETS provider and can continue
setting up DRealty. Make sure "Enabled" checkbox for the connection is checked on.

Each connection can have resources (Property, Agent, Office, OpenHouse, etc.) configured here.
For example, clicking 'Configure Property' will ask you to map the Resource, and then
configuring each of the property types. This interface lets you map all the fields
necessary from RETS data to Drupal fields.

### Mapping RETS Resources (Units / Listings / Agents / Offices , etc.):

- Go to "DRealty" -> pick a connection and click "MANAGE" (`/admin/drealty/connection/[connection_name]`).
You will see all the available RETS Classes for each Resource available for mapping.
- To enable import of specific resource class it has to be "Enabled" there. If it's not checked, records for
this Class type won't be imported. 
    - Pick a Class to map, click "Map Entities" and pick Drupal entity to map it
      into. SAVE configuration.
    - After saving, there will be "Configure" button available to proceed with configuration
      of the top level mappings. Fill in unique fields mappings, pick search query,
      whether to process Media data for this Resource, etc. Save. 
        - The ID, KEY and STATUS field are important and will affect the 
       success of your data querying.
        - MODIFIED FIELD - is also important now as CRON / Drush does rely on 
          it when detecting TimeDifference setting for incremental updates.
        - for "RETS IMPORT QUERY OPTIONS" - go with `Manually Supplied query`,
          which you can enter as a set of various conditions, e.g. `(StandardStatus=|10000069142),(County=|10000003357)`.
    - Let's say, you mapped a Class to some NODE bundle. Configuration of the
      field mappings for Drupal - RETS connection on this node can be done under
      "Structure -> Content types -> Node Bundle -> Manage fields" (`/admin/structure/types/manage/[node_bundle]/fields`).
      For each field you will be able to configure mapping, once you edit the field,
      under "DRealty" fieldset.
        - Pick "PROPERTY" of the Drupal field to save the data into. For simple 
          fields it usually will be "value" property. For fields like Address type, 
          for example, it can have other subfields.
        - Then select RETS "Class" from the dropdown
        - Then select "Field" from that class in RETS that will be imported into Drupal.
        - Click "ADD". Once done with everything, save the Node.
    - MEDIA RETS Resources can be mapped into Drupal core Media entity. Add mapping
      config through an entity reference field (to Media entity) on the node. Make 
      sure "Process Media?" is checked for this specific Rets Class - Drupal node 
      mapping config (on `/admin/structure/types/manage/[node_bundle]/drealty/[class_mapping_identifier]/manage`).
    

--------------------------------------------------------------------
IMPORT - Ultimate Cron method -- NOT IMPLEMENTED YET
--------------------------------------------------------------------
@TODO

--------------------------------------------------------------------
BATCH FULL IMPORT
--------------------------------------------------------------------
- Go to "DRealty" -> pick a connection and click "MANAGE" (`/admin/drealty/connection/[connection_name]`).
  You will see connection info at the top, and a "Full data import" button there.
  
--------------------------------------------------------------------
DRUSH IMPORT
--------------------------------------------------------------------

After enabling and setting up your DRealty connection(s), getting the site
populated with data is a matter of creating crontab jobs with some basic
Drush commands.

#### --------------------    Workflow:    -------------
DRUSH IMPORT gets data based on the conditions you have configured for each
particular Resource Class (for example, under DRealty > [Your Specific Connection] > Edit Connection)


`drush rets-import`  (without any options) is not running import with
INCREMENTAL update - it imports all of the data available for this mapping. You 
should trigger it first to populate the initial set of records on your site.

After the initial import you can configure periodic Drush invocation with incremental option.
For DRUSH to respect the Modified value of the RETS data, add Drush option `drush rets-import --increment=TRUE`

If you enabled MEDIA mapping for resources, you'll need to also run Drush calls for
incremental Media updates afterwards:
`drush rets-import-media --increment=TRUE` - this creates Drupal queue to process
`drush rets-process-media` - will take care of processing generated queue items 
