<?php

namespace Drupal\drealty\Controller;

use Drupal\drealty\Entity\DrealtyClassInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

// Use Drupal\Component\Utility\Xss;.

/**
 * Defines a route controller for watches autocomplete form elements.
 */
class DrealtyAutoCompleteController extends ControllerBase {

  /**
   * The dRealty Class storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $classStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->classStorage = $entity_type_manager->getStorage('drealty_class');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Handler for autocomplete request based on DRealty Class Fields.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The Request instance.
   * @param \Drupal\drealty\Entity\Connection $drealty_connection
   *   RETS Connection.
   * @param \Drupal\drealty\Entity\DrealtyClass $drealty_class
   *   RETS Class.
   */
  public function handleClassFieldsAutocomplete(Request $request, $drealty_connection, $drealty_class) {
    $match = $request->query->get('q');
    $matches = [];
    if (!$match) {
      return new JsonResponse($matches);
    }
    // $match = Xss::filter($match);
    if (!$drealty_class instanceof DrealtyClassInterface) {
      $drealty_class = $this->classStorage->load($drealty_class);
    }

    if ($drealty_class && !empty($fields = $drealty_class->getFields())) {
      $find = preg_grep('~' . $match . '~i', array_column($fields, 'systemname'));
      if (!empty($find)) {
        foreach ($find as $i => $v) {
          $matches[] = [
            'value' => $v,
            'label' => $v,
          ];
        }

      }
    }
    return new JsonResponse($matches);
  }

}
