<?php

namespace Drupal\drealty\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\drealty\Entity\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Render\FormattableMarkup;

/**
 * Provides route responses for DRealty Connection entity.
 */
class DrealtyConnectionEntityController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->renderer = $container->get('renderer');
    return $instance;
  }

  /**
   * Route title callback.
   *
   * @return string
   *   The title for the DRealty Connection form.
   */

  /**
   * (Connection $drealty_connection) {
   */
  public function title() {

    $request = \Drupal::request()->getRequestUri();
    $route = \Drupal::routeMatch();
    $route_name = $route->getRouteName();
    $route_params = $route->getParameters()->all();
    if (!empty($route_params['drealty_connection'])) {
      $entity = ($route_params['drealty_connection'] instanceof EntityInterface) ? $route_params['drealty_connection'] : $this->entityTypeManager()
        ->getStorage('drealty_connection')
        ->load($route_params['drealty_connection']);
    }

    if (!empty($route_params['drealty_class'])) {
      $entity2 = ($route_params['drealty_class'] instanceof EntityInterface) ? $route_params['drealty_class'] : $this->entityTypeManager()
        ->getStorage('drealty_class')
        ->load($route_params['drealty_class']);

      if ($route_name == 'entity.drealty_class.entities') {
        return $this->t('%conn connection: %label class entity mappings',
          [
            '%label' => strtoupper($entity2->label()),
            '%conn' => strtoupper($entity2->getConnectionId()),
          ]);
      }

      return $this->t('Manage %label class', [
        '%label' => strtoupper($entity2->label()),
        '%conn' => strtoupper($entity2->getConnectionId()),
      ]);
    }

    if (!empty($entity)) {
      if ($route_name == 'entity.drealty_class.collection') {
        return $this->t('%label connection overview', [
          '%label' => strtoupper($entity->label()),
        ]);
      }
      return $this->t('Manage %label connection', [
        '%label' => strtoupper($entity->label()),
      ]);
    }
    return '';
  }

  /**
   *
   */
  public function viewEntityCotroller(Connection $drealty_connection, $entity_type) {
    $render['intro_data'] = [
      '#type' => 'table',
      '#attributes' => ['style' => 'border: collapse;'],
      '#responsive' => TRUE,
      '#rows' => [
        [
          'data' => [
            ['data' => new FormattableMarkup('<strong>Login URL</strong>', [])],
            ['data' => $drealty_connection->getLoginUrl()],
          ],
          'style' => 'border: none !important;',
        ],
        [
          'data' => [
            ['data' => new FormattableMarkup('<strong>Status</strong>', [])],
            ['data' => $drealty_connection->status() ? 'Enabled' : 'Disabled'],
          ],
          'style' => 'border: none !important;',
        ],
        [
          'data' => [
            ['data' => new FormattableMarkup('<strong>RETS Version</strong>', [])],
            ['data' => $drealty_connection->getVersion()],
          ],
          'style' => 'border: none !important;',
        ],
        [
          'data' => [
            ['data' => new FormattableMarkup('<strong>RETS Username</strong>', [])],
            ['data' => $drealty_connection->getUsername()],
          ],
          'style' => 'border: none !important;',
        ],
      ],
    ];
    $classes = $this->entityTypeManager()
      ->getListBuilder($entity_type)
      ->render();
    if (!empty($classes)) {
      $render['intro_extra'] = [
        '#markup' => '<br><br><h2>Connection RETS Classes</h2>',
      ];

      $render += $classes;
    }
    return $render;
  }

}
