<?php

namespace Drupal\drealty;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Url;

/**
 * Provides a listing of drealty classes.
 */
class DrealtyClassListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['label'] = $this->t('Label');
    $header['id'] = $this->t('Machine name');
    $header['status'] = $this->t('Status');
    return $header + parent::buildHeader();
  }

  /**
   * Loads entity IDs using a pager sorted by the entity id.
   *
   * @return array
   *   An array of entity IDs.
   */
  protected function getEntityIds() {
    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->sort($this->entityType->getKey('id'));

    if ($conn = \Drupal::routeMatch()->getParameter('drealty_connection')) {
      $query->condition('connection', $conn->id());
    }

    // Only add the pager if a limit is specified.
    if ($this->limit) {
      $query->pager($this->limit);
    }
    return $query->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    /** @var \Drupal\Core\Config\Entity\ConfigEntityInterface $entity */
    $operations = parent::getDefaultOperations($entity);

    if (\Drupal::currentUser()
      ->hasPermission('administer drealty connections')) {

      $operations['entities'] = [
        'title' => t('Map Entities'),
        'url' => Url::fromRoute('entity.drealty_class.entities', [
          'drealty_connection' => $entity->getConnectionId(),
          'drealty_class' => $entity->id(),
        ]),
        'weight' => 50,
      ];
    }

    // Check if we're on a mapped entity (e.g. node):
    $bundle_entity_type_id = \Drupal::routeMatch()
      ->getParameter('bundle_entity_type_id');
    // Hide EDIT operation otherwise:
    if (empty($bundle_entity_type_id)) {
      unset($operations["edit"]);
      return $operations;
    }

    // If on an entity route, hide EDIT/ENTITIES operations:
    unset($operations["entities"]);
    unset($operations["delete"]);

    $entity_type_id = \Drupal::routeMatch()->getParameter('entity_type_id');
    $bundle = \Drupal::routeMatch()->getParameter($bundle_entity_type_id);
    $route_parameters = [
      'entity_type_id' => $entity_type_id,
      'bundle_entity_type_id' => $bundle_entity_type_id,
      $bundle_entity_type_id => $bundle->id(),
      'drealty_class' => $entity->id(),
    ];
    // If on an entity route, update EDIT operation URL so that this path is
    // working "/admin/structure/types/manage/[**node|entity bundle**]/drealty":
    $operations["edit"]["url"] = Url::fromRoute(
      "entity.{$entity_type_id}.drealty_class_edit_form",
      $route_parameters
    );

    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /** @var \Drupal\drealty\DrealtyClassInterface $entity */
    $row['label'] = !empty($entity->resource) ? $entity->resource . ': ' . $entity->label() : $entity->label();
    $row['id'] = $entity->id();
    $row['status'] = $entity->status() ? $this->t('Enabled') : $this->t('Disabled');
    return $row + parent::buildRow($entity);
  }

}
