<?php

namespace Drupal\drealty\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;

/**
 * Defines the DRealty MLS connection entity type.
 *
 * @ConfigEntityType(
 *   id = "drealty_connection",
 *   label = @Translation("DRealty connection"),
 *   label_collection = @Translation("DRealty connections"),
 *   label_singular = @Translation("DRealty connection"),
 *   label_plural = @Translation("DRealty connections"),
 *   label_count = @PluralTranslation(
 *     singular = "@count DRealty connection",
 *     plural = "@count DRealty connections",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\drealty\ConnectionListBuilder",
 *     "form" = {
 *       "add" = "Drupal\drealty\Form\ConnectionForm",
 *       "edit" = "Drupal\drealty\Form\ConnectionForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *       "update-resources" =
 *   "Drupal\drealty\Form\ConnectionUpdateResourcesForm"
 *     }
 *   },
 *   config_prefix = "drealty_connection",
 *   admin_permission = "administer drealty connections",
 *   links = {
 *     "collection" = "/admin/drealty/connection",
 *     "add-form" = "/admin/drealty/connection/add",
 *     "edit-form" = "/admin/drealty/connection/{drealty_connection}/edit",
 *     "delete-form" = "/admin/drealty/connection/{drealty_connection}/delete",
 *     "update-resources-form" =
 *   "/admin/drealty/connection/{drealty_connection}/update-resources"
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "description",
 *     "login_url",
 *     "username",
 *     "password",
 *     "ua_string",
 *     "ua_password",
 *     "version",
 *     "use_interealty_auth",
 *     "force_basic_auth",
 *     "use_compression",
 *     "disable_encoding_fix",
 *     "debug_mode",
 *     "nomap_mode",
 *     "format",
 *     "resources",
 *     "import_modified_span"
 *   }
 * )
 */
class Connection extends ConfigEntityBase implements ConnectionInterface {

  /**
   * The connection ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The connection label.
   *
   * @var string
   */
  protected $label;

  /**
   * The connection status.
   *
   * @var bool
   */
  protected $status;

  /**
   * The connection description.
   *
   * @var string
   */
  protected $description;

  /**
   * The connection LoginUrl.
   *
   * @var string
   */
  protected $login_url;

  /**
   * The connection Username.
   *
   * @var string
   */
  protected $username;

  /**
   * The connection Password.
   *
   * @var string
   */
  protected $password;

  /**
   * The connection Password.
   *
   * @var string
   */
  protected $ua_string;

  /**
   * The connection Password.
   *
   * @var string
   */
  protected $ua_password;

  /**
   * The connection Password.
   *
   * @var string
   */
  protected $version;

  /**
   * The connection Password.
   *
   * @var bool
   */
  protected $use_interealty_auth;

  /**
   * The connection Password.
   *
   * @var bool
   */
  protected $force_basic_auth;

  /**
   * The connection use compression.
   *
   * @var bool
   */
  protected $use_compression;

  /**
   * The connection disable EncodingFix.
   *
   * @var bool
   */
  protected $disable_encoding_fix;

  /**
   * The connection debug mode.
   *
   * @var bool
   */
  protected $debug_mode;

  /**
   * The connection "No mapping" mode.
   *
   * @var bool
   */
  protected $nomap_mode;

  /**
   * The connection Format.
   *
   * @var string
   */
  protected $format;

  /**
   * The connection Format.
   *
   * @var int
   */
  public $import_modified_span;

  /**
   * RETS fetcher.
   *
   * @var \Drupal\drealty\RetsFetcher
   */
  protected $fetcher;

  /**
   * RETS Resources objects array.
   *
   * @var \Drupal\drealty\Entity\ResourceInterface[]
   */
  protected $resources = [];

  /**
   * {@inheritdoc}
   */
  public function __construct(array $values, $entity_type) {
    parent::__construct($values, $entity_type);
  }

  /**
   * {@inheritdoc}
   */
  public function getLoginUrl(): string {
    return (string) $this->login_url;
  }

  /**
   * {@inheritdoc}
   */
  public function setLoginUrl(string $login_url): void {
    $this->login_url = $login_url;
  }

  /**
   * {@inheritdoc}
   */
  public function getUsername(): string {
    return (string) $this->username;
  }

  /**
   * {@inheritdoc}
   */
  public function setUsername(string $username): void {
    $this->username = $username;
  }

  /**
   * {@inheritdoc}
   */
  public function getPassword(): string {
    return (string) $this->password;
  }

  /**
   * {@inheritdoc}
   */
  public function setPassword(string $password): void {
    $this->password = $password;
  }

  /**
   * {@inheritdoc}
   */
  public function getUaString(): string {
    return (string) $this->ua_string;
  }

  /**
   * {@inheritdoc}
   */
  public function setUaString(string $ua_string): void {
    $this->ua_string = $ua_string;
  }

  /**
   * {@inheritdoc}
   */
  public function getUaPassword(): string {
    return (string) $this->ua_password;
  }

  /**
   * {@inheritdoc}
   */
  public function setUaPassword(string $ua_password): void {
    $this->ua_password = $ua_password;
  }

  /**
   * {@inheritdoc}
   */
  public function getVersion(): string {
    return (string) $this->version;
  }

  /**
   * {@inheritdoc}
   */
  public function setVersion(string $version): void {
    $this->version = $version;
  }

  /**
   * {@inheritdoc}
   */
  public function isUseInterealtyauth(): bool {
    return (bool) $this->use_interealty_auth;
  }

  /**
   * {@inheritdoc}
   */
  public function setUseInterealtyauth(bool $use_interealty_auth): void {
    $this->use_interealty_auth = $use_interealty_auth;
  }

  /**
   * {@inheritdoc}
   */
  public function isForceBasicauth(): bool {
    return (bool) $this->force_basic_auth;
  }

  /**
   * {@inheritdoc}
   */
  public function setForceBasicauth(bool $force_basic_auth): void {
    $this->force_basic_auth = $force_basic_auth;
  }

  /**
   * {@inheritdoc}
   */
  public function isUseCompression(): bool {
    return (bool) $this->use_compression;
  }

  /**
   * {@inheritdoc}
   */
  public function setUseCompression(bool $use_compression): void {
    $this->use_compression = $use_compression;
  }

  /**
   * {@inheritdoc}
   */
  public function isDisableEncodingfix(): bool {
    return (bool) $this->disable_encoding_fix;
  }

  /**
   * {@inheritdoc}
   */
  public function setDisableEncodingfix(bool $disable_encoding_fix): void {
    $this->disable_encoding_fix = $disable_encoding_fix;
  }

  /**
   * {@inheritdoc}
   */
  public function isDebugMode(): bool {
    return (bool) $this->debug_mode;
  }

  /**
   * {@inheritdoc}
   */
  public function setDebugMode(bool $debug_mode): void {
    $this->debug_mode = $debug_mode;
  }

  /**
   * {@inheritdoc}
   */
  public function isNomapMode(): bool {
    return (bool) $this->nomap_mode;
  }

  /**
   * {@inheritdoc}
   */
  public function setNomapMode(bool $nomap_mode): void {
    $this->nomap_mode = $nomap_mode;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormat(): string {
    return $this->format;
  }

  /**
   * {@inheritdoc}
   */
  public function setFormat(string $format): void {
    $this->format = $format;
  }

  /**
   * Returns the RETS fetcher.
   *
   * @return \Drupal\drealty\RetsFetcherInterface
   *   RETS fetcher.
   */
  public function getFetcher() {
    if (!isset($this->fetcher)) {
      $this->fetcher = \Drupal::service('drealty.fetcher');
    }
    return $this->fetcher;
  }

  /**
   * @return bool|\Drupal\drealty\RetsSession
   *   RETS session status: started or not.
   */
  public function connect() {
    return $this->getFetcher()->connectRetsServer($this);
  }

  /**
   * Get remote resources from RETS server.
   */
  public function getRemoteResources() {
    return $this->getFetcher()->getRETSResourcesStructured($this);
  }

  /**
   * Get remote classes from RETS server.
   */
  public function getRemoteClasses() {
    return $this->connect()->GetClassesMetadata('Property');
  }

  /**
   * Get resources stored on this connection.
   */
  public function getResources() {
    return $this->resources;
  }

  /**
   * Clean up any remaining Drupal data about this connection.
   */
  public function cleanupConnectionConfig() {
    \Drupal::database()
      ->delete('config')
      ->condition('name', 'drealty.drealty_class.' . $this->id() . '.', 'like')
      ->execute();
    \Drupal::database()->delete('key_value')
      ->condition('collection', '%.drealty_%', 'like')
      ->condition('value', 'drealty.drealty_%.' . $this->id() . '%', 'like')
      ->execute();

    // Find mapped fields config (have 'drealty.mapping' value not empty):
    $fields_config = \Drupal::entityQuery('field_config')
      ->condition('third_party_settings.drealty.mapping.' . $this->id(), '', 'IS NOT NULL')
      ->accessCheck(FALSE)
      ->execute();
    $fields_config = \Drupal::entityTypeManager()
      ->getStorage('field_config')
      ->loadMultiple($fields_config);

    foreach ($fields_config as $config) {
      $mapping = $config->getThirdPartySetting('drealty', 'mapping', []);
      // Clean up removed connection relation and update:
      unset($mapping[$this->id()]);
      $config->setThirdPartySetting('drealty', 'mapping', $mapping)->save();
    }

  }

  /**
   * Resource setter.
   */
  public function setResources($resources) {
    $this->resources = $resources;
    return $this;
  }

  /**
   * Helper function to convert objects data to array output.
   */
  public function toArray() {
    $result = parent::toArray();
    foreach ($result['resources'] as $key => $value) {
      $result['resources'][$key] = is_array($value) ? $value : $value->toArray();
    }
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities) {
    foreach ($entities as $entity) {
      $entity->cleanupConnectionConfig();
    }
  }

  /**
   * Find class objects for this specific class.
   */
  public function findClassObjects($class) {
    $objects = [];
    $cid = str_replace('.', '__', $class->id());
    if (!empty($this->resources) && !empty($this->resources[$class->resource])
      && !empty($this->resources[$class->resource]['classes'][$cid]['objects'])) {
      $objects = $this->resources[$class->resource]['classes'][$cid]['objects'];
    }
    return $objects;
  }

}
