<?php

namespace Drupal\drealty\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Config\Entity\ConfigEntityInterface;

/**
 * Defines the drealty class entity type.
 *
 * @ConfigEntityType(
 *   id = "drealty_class",
 *   label = @Translation("DRealty class"),
 *   label_collection = @Translation("DRealty classes"),
 *   label_singular = @Translation("DRealty class"),
 *   label_plural = @Translation("DRealty classes"),
 *   label_count = @PluralTranslation(
 *     singular = "@count DRealty class",
 *     plural = "@count DRealty classes",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\drealty\DrealtyClassListBuilder",
 *     "form" = {
 *       "add" = "Drupal\drealty\Form\DrealtyClassForm",
 *       "edit" = "Drupal\drealty\Form\DrealtyClassForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *       "entities" = "Drupal\drealty\Form\EntitiesForm"
 *     }
 *   },
 *   config_prefix = "drealty_class",
 *   admin_permission = "administer drealty classes",
 *   links = {
 *     "collection" = "/admin/drealty/connection/{drealty_connection}",
 *     "add-form" = "/admin/drealty/connection/{drealty_connection}/class/add",
 *     "edit-form" =
 *   "/admin/drealty/connection/{drealty_connection}/class/{drealty_class}/edit",
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "description",
 *     "connection",
 *     "resource",
 *     "systemname",
 *     "standardname",
 *     "visiblename",
 *     "bundle",
 *     "lifetime",
 *     "enabled",
 *     "configured",
 *     "lastupdate",
 *     "media_lastupdate",
 *     "status_values",
 *     "process_images",
 *     "image_chunk_size",
 *     "query_type",
 *     "chunk_size",
 *     "override_status_query_text",
 *     "offset_field",
 *     "offset_amount",
 *     "offset_max",
 *     "expired_handler",
 *     "rets_timestamp_field",
 *     "fields",
 *     "objects",
 *     "drealty_mapping"
 *   }
 * )
 */
class DrealtyClass extends ConfigEntityBase implements DrealtyClassInterface {

  /**
   * The drealty class ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The drealty class label.
   *
   * @var string
   */
  protected $label;

  /**
   * The RETS class status.
   *
   * @var bool
   */
  protected $status;

  /**
   * The RETS class description.
   *
   * @var string
   */
  protected $description;


  /**
   * Array of RETS fields.
   *
   * @var \Drupal\drealty\Entity\FieldInterface[]
   */
  protected $fields = [];

  /**
   * Array of RETS objects.
   *
   * @var \Drupal\drealty\Metadata\RetsObject[]
   */
  protected $objects = [];

  /**
   * RETS connection.
   *
   * @var \Drupal\drealty\Entity\ConnectionInterface
   */
  protected $connection;

  /**
   * Should process images.
   *
   * @var bool
   */
  protected $process_images;

  /**
   * Image process size.
   *
   * @var int
   */
  protected $image_chunk_size;

  /**
   * RETS mapping config array.
   *
   * @var array
   */
  public $drealty_mapping = [];

  /**
   * {@inheritdoc}
   */
  public function __construct(array $values, $entity_type) {
    parent::__construct($values, $entity_type);
  }

  /**
   * Fields getter.
   */
  public function getFields() {
    return $this->fields;
  }

  /**
   * Fields setter.
   */
  public function setFields($fields) {
    $this->fields = $fields;
    return $this;
  }

  /**
   * Objects setter.
   */
  public function setObjects($objects) {
    $this->objects = $objects;
    return $this;
  }

  /**
   * Objects getter.
   */
  public function getObjects() {
    return $this->objects;
  }

  /**
   *
   */
  // Public function getImageChunkSize() {
  //    return $this->image_chunk_size;
  //  }.

  /**
   *
   */
  // Public function setImageChunkSize($image_chunk_size) {
  //    $this->image_chunk_size = $image_chunk_size;
  //    return $this;
  //  }.

  /**
   *
   */
  public function getEnableImageProcessing() {
    return $this->process_images;
  }

  /**
   *
   */
  public function setEnableImageProcessing($process_images) {
    $this->process_images = $process_images;
    return $this;
  }

  /**
   * Convert into array.
   */
  public function toArray() {
    $result = parent::toArray();
    foreach ($result['fields'] as $key => $value) {
      $result['fields'][$key] = is_array($value) ? $value : $value->toArray();
    }
    return $result;
  }

  /**
   * Retrieve connection ID.
   */
  public function getConnectionId() {
    return $this->connection;
  }

  /**
   * Helper callback for uasort() to sort configuration entities by weight and label.
   */
  public static function sort(ConfigEntityInterface $a, ConfigEntityInterface $b) {
    if ($a->status() == $b->status()) {
      return strnatcasecmp($a->id(), $b->id());
    }
    else {
      return $a->status() ? -1 : 1;
    }
  }

}
