<?php

namespace Drupal\drealty\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines the DRealty resource entity type.
 *
 * @ConfigEntityType(
 *   id = "drealty_resource",
 *   label = @Translation("DRealty resource"),
 *   label_collection = @Translation("DRealty resources"),
 *   label_singular = @Translation("DRealty resource"),
 *   label_plural = @Translation("DRealty resources"),
 *   label_count = @PluralTranslation(
 *     singular = "@count drealty resource",
 *     plural = "@count drealty resources",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\drealty\ResourceListBuilder",
 *     "form" = {
 *       "add" = "Drupal\drealty\Form\ResourceForm",
 *       "edit" = "Drupal\drealty\Form\ResourceForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm"
 *     }
 *   },
 *   config_prefix = "drealty_resource",
 *   admin_permission = "administer drealty resource",
 *   links = {
 *     "add-form" =
 *   "/admin/drealty/connection/{drealty_connection}/resource/add",
 *     "edit-form" =
 *   "/admin/drealty/connection/{drealty_connection}/resource/{drealty_resource}/edit",
 *     "delete-form" =
 *   "/admin/drealty/connection/{drealty_connection}/resource/{drealty_resource}/delete"
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "description",
 *     "lastupdate",
 *     "keyfield",
 *     "classes"
 *   }
 * )
 */
class Resource extends ConfigEntityBase implements ResourceInterface {

  /**
   * The DRealty resource ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The drealty resource label.
   *
   * @var string
   */
  protected $label;

  /**
   * The drealty resource status.
   *
   * @var bool
   */
  protected $status;

  /**
   * The drealty_resource description.
   *
   * @var string
   */
  protected $description;

  /**
   * The last update.
   *
   * @var string
   */
  protected $lastupdate;

  /**
   * The keyfield.
   *
   * @var string
   */
  protected $keyfield;

  /**
   * @var \Drupal\drealty\Entity\DrealtyClassInterface[]
   */
  protected $classes = [];

  /**
   * {@inheritdoc}
   */
  public function __construct(array $values, $entity_type) {
    parent::__construct($values, $entity_type);
    $resource_storage = $this->entityTypeManager()->getStorage('drealty_class');
    foreach ($this->classes as $key => $value) {
      $this->classes[$key] = $resource_storage->load($value['id']);
    }
  }

  /**
   *
   */
  public function toArray() {
    $result = parent::toArray();
    foreach ($result['classes'] as $key => $value) {
      $result['classes'][$key] = is_array($value) ? $value : $value->toArray();
    }
    return $result;
  }

  /**
   *
   */
  public function getClasses() {
    return $this->classes;
  }

  /**
   *
   */
  public function setClasses($classes) {
    $this->classes = $classes;
    return $this;
  }

}
