<?php

namespace Drupal\drealty\Metadata;

use Drupal\drealty\Entity\ConnectionInterface;
use PHRETS\Models\Metadata\Table;

/**
 * RETS Metadata for Class fields.
 */
class Field {

  /**
   * RETS field ID.
   *
   * @var string
   */
  public $fid;

  /**
   * RETS connection ID.
   *
   * @var string
   */
  public $conid;

  /**
   * RETS resource type.
   *
   * @var string
   */
  public $resource;

  /**
   * RETS SystemName field.
   *
   * @var string
   */
  public $systemname;

  /**
   * RETS StandardName field.
   *
   * @var string
   */
  public $standardname;

  /**
   * RETS long name.
   *
   * @var string
   */
  public $longname;

  /**
   * RETS data type.
   *
   * @var string
   */
  public $datatype;

  /**
   * RETS field max length.
   *
   * @var int|string
   */
  public $max_length;

  /**
   * RETS field precision.
   *
   * @var int|string
   */
  public $field_precision;

  /**
   * RETS interpretation.
   *
   * @var string
   */
  public $interpretation;

  /**
   * RETS lookup name.
   *
   * @var string
   */
  public $lookupname;

  /**
   * RETS class identifier.
   *
   * @var string
   */
  public $class;

  /**
   * RETS last updated date.
   *
   * @var null|int
   */
  public $lastupdate;

  /**
   * Constructor.
   */
  public function __construct(ConnectionInterface $connection, Table $field) {
    $this->conid = $connection->id();
    $this->resource = $field->getResource();
    $this->lastupdate = strtotime($field->getClassDate());
    $this->systemname = $field->getSystemName();
    $this->standardname = $field->getStandardName();
    $this->longname = $field->getLongName();
    $this->datatype = $field->getDataType();
    $this->max_length = $field->getMaximumLength();
    $this->field_precision = $field->getPrecision() === '' ? 0 : $field->getPrecision();
    $this->interpretation = $field->getInterpretation();
    $this->class = $field->getClass();
    $this->lastupdate = strtotime("now");
    $this->lookupname = $field->getLookupName();
  }

}
