<?php

namespace Drupal\drealty\Metadata;

use Drupal\drealty\Entity\ConnectionInterface;
use PHRETS\Models\Metadata\BaseObject;

/**
 * Metadata for RETS Resource.
 */
class RetsObject {

  /**
   * RETS connection ID.
   *
   * @var string
   */
  public $connection;

  /**
   * RETS resource type.
   *
   * @var string
   */
  public $resource;

  /**
   * RETS Metadata Entry ID.
   *
   * @var string
   */
  public $metadataentryid;

  /**
   * RETS Description field.
   *
   * @var string
   */
  public $description;

  /**
   * RETS visible name field.
   *
   * @var string
   */
  public $visiblename;

  /**
   * RETS object type.
   *
   * @var string
   */
  public $objecttype;

  /**
   * RETS Description field.
   *
   * @var string
   */
  public $objecttimestamp;

  /**
   * RETS count of attached objects.
   *
   * @var string
   */
  public $objectcount;

  /**
   * Mime type.
   *
   * @var string
   */
  public $mimetype;

  /**
   * Constructor.
   */
  public function __construct(ConnectionInterface $connection, BaseObject $object) {
    $this->connection = $connection->id();
    $this->description = $object->getDescription();
    $this->resource = $object->getResource();
    $this->metadataentryid = $object->getMetadataEntryID();
    $this->objectcount = $object->getObjectCount();
    $this->objecttype = $object->getObjectType();
    $this->objecttimestamp = $object->getObjectTimeStamp();
    $this->visiblename = $object->getVisibleName();
    $this->mimetype = $object->getMIMEType();
  }

  /**
   * Convert RETS object data into array.
   */
  public function valuesArray() {
    return [
      'connection' => $this->connection,
      'description' => $this->description,
      'resource' => $this->resource,
      'metadataentryid' => $this->metadataentryid,
      'objectcount' => $this->objectcount,
      'objecttype' => $this->objecttype,
      'objecttimestamp' => $this->objecttimestamp,
      'visiblename' => $this->visiblename,
      'mimetype' => $this->mimetype,
    ];
  }

}
