<?php

namespace Drupal\drealty\Plugin\QueueWorker;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\drealty\RetsFetcherInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Process a queue of drealty import.
 *
 * @QueueWorker(
 *   id = "drealty_import",
 *   title = @Translation("DRealty import"),
 *   cron = {"time" = 60}
 * )
 */
class DrealtyImport extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\drealty\RetsFetcher
   */
  protected $fetcher;

  /**
   * Constructs a new class instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, RetsFetcherInterface $fetcher) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->fetcher = $fetcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('drealty.fetcher')
    );
  }

  /**
   * Updates a DRealty's items.
   *
   * @QueueWorker(
   *   id = "drealty_import",
   *   title = @Translation("DRealty import"),
   *   cron = {"time" = 60}
   * )
   */
  public function processItem($data) {
    $retsService = \Drupal::service('drealty.fetcher');
    try {
      $entity_type = $data['entity_type'];
      $bundle = $data['bundle'];
      $storage = $this->entityTypeManager->getStorage($entity_type);

      $listing = $retsService->entityExists($data['entity_type'], $data['remote_id'], NULL, $data['class'], $data['bundle']);
      if (!($listing)) {
        $values = [
          $storage->getEntityType()->getKey('bundle') => $bundle,
          $storage->getEntityType()->getKey('status') => 1,
          $storage->getEntityType()->getKey('owner') => 1,
          $storage->getEntityType()
            ->getKey('label') => "MLS Resource data: " . $data['remote_id'],
        ];
        $listing = $storage->create($values);
        $listing->set('rets_class', $data['class']);
      }

      // , $connection, $class, $data['mapping']);
      $retsService->update_single_listing($listing, $data);
    }
    catch (\Exception $e) {
      $t = 1;
      // watchdog("drealty_process_queue", $e->getMessage(), array(), WATCHDOG_ERROR);
      //      drupal_set_message($e->getMessage(), 'error');.
    }
  }

}
