<?php

namespace Drupal\drealty\Routing;

use Drupal\Core\Routing\RouteSubscriberBase;
use Symfony\Component\Routing\RouteCollection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RoutingEvents;
use Drupal\drealty\Form\ImportForm;
use Symfony\Component\Routing\Route;

/**
 * Subscriber for Field UI routes.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a RouteSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function alterRoutes(RouteCollection $collection) {
    \Drupal::logger('drealty_router')->warning('alter routes triggered');
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      if ($route_name = $entity_type->get('field_ui_base_route')) {
        // Try to get the route from the current collection.
        if (!$entity_route = $collection->get($route_name)) {
          continue;
        }
        $path = $entity_route->getPath();
        $options = $entity_route->getOptions();
        if ($bundle_entity_type = $entity_type->getBundleEntityType()) {
          $options['parameters'][$bundle_entity_type] = [
            'type' => 'entity:' . $bundle_entity_type,
          ];
        }
        else {
          continue;
        }

        $defaults = [
          'entity_type_id' => $entity_type_id,
          'bundle_entity_type_id' => $bundle_entity_type,
        ];

        // For "/admin/structure/types/manage/[**node|entity bundle**]/drealty":
        $route = new Route(
          "$path/drealty",
          [
            '_entity_list' => 'drealty_connection',
          ] + $defaults,
          ['_permission' => 'administer drealty connections'],
          $options
        );
        $collection->add("entity.{$entity_type_id}.drealty_class_list", $route);

        $route = new Route(
          "$path/drealty/import",
          [
            '_title' => 'DRealty Import',
            '_form' => ImportForm::class,
          ] + $defaults,
          ['_permission' => 'administer site configuration'],
          $options
        );
        $collection->add("entity.{$entity_type_id}.drealty_import", $route);

        // For "/admin/structure/types/manage/[**node|entity bundle**]/drealty/[**drealty class**]/manage":
        $route = new Route(
          "$path/drealty/{drealty_class}/manage",
          ['_entity_form' => 'drealty_class.edit'] + $defaults,
          ['_permission' => 'administer drealty classes'],
          // ['_access' => 'TRUE'],
          $options
        );
        $collection->add("entity.{$entity_type_id}.drealty_class_edit_form", $route);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = parent::getSubscribedEvents();
    $events[RoutingEvents::ALTER] = ['onAlterRoutes', -100];
    return $events;
  }

}
